package api

import (
	"encoding/json"
	"fmt"
	"github.com/gorilla/mux"
	"gorm.io/gorm"
	"io"
	"net/http"
	"os"
	"sonarqube-badge/config"
	"sonarqube-badge/router/middlewares"
	"sonarqube-badge/security"
	"sonarqube-badge/store"
)

type setProjectBadgePayload struct {
	Token string `json:"token"`
}

func getProjectBadge(w http.ResponseWriter, r *http.Request) {
	vars := mux.Vars(r)
	projectName := vars["name"]
	metric := vars["metric"]

	print(projectName)

	project := store.SQProject{}
	ctx := r.Context()
	cfg := ctx.Value("config").(config.Config)
	ctx.Value("db").(*gorm.DB).First(&project, "project_name = ?", projectName)
	if project.ID == 0 {
		w.WriteHeader(http.StatusNotFound)
		w.Write([]byte("project not found"))
		return
	}

	token := security.DecryptAES(cfg.Secret, project.Token)
	fmt.Printf("%s", token)

	url := fmt.Sprintf("%s/api/project_badges/measure?metric=%s&project=%s&token=%s", cfg.SqHost, metric, projectName, token)
	request, err := http.Get(url)
	if err != nil {
		w.WriteHeader(http.StatusInternalServerError)
		os.Stderr.Write([]byte(err.Error()))
		w.Write([]byte("internal server error"))
		return
	}
	bytes, err := io.ReadAll(request.Body)
	if err != nil {
		w.WriteHeader(http.StatusInternalServerError)
		os.Stderr.Write([]byte(err.Error()))
		w.Write([]byte("internal server error"))
		return
	}
	w.Header().Set("Content-Type", "image/svg+xml")
	w.WriteHeader(request.StatusCode)
	w.Write(bytes)
}

func setProjectBadge(w http.ResponseWriter, r *http.Request) {
	var payload setProjectBadgePayload
	vars := mux.Vars(r)
	projectName := vars["name"]

	err := json.NewDecoder(r.Body).Decode(&payload)
	if err != nil {
		w.WriteHeader(http.StatusBadRequest)
		os.Stderr.Write([]byte(err.Error()))
		w.Write([]byte("invalid payload"))
		return
	}

	ctx := r.Context()
	db := ctx.Value("db").(*gorm.DB)

	if !middlewares.CheckAuth(ctx, r) {
		w.WriteHeader(http.StatusUnauthorized)
		w.Write([]byte("unauthorized"))
		return
	}

	project := store.SQProject{}

	db.First(&project, "project_name = ?", projectName)

	project.ProjectName = projectName
	project.Token = security.EncryptAES(ctx.Value("config").(config.Config).Secret, payload.Token)

	db.Save(&project)

	w.WriteHeader(http.StatusOK)
	w.Write([]byte("Successfully added"))
}

func ProjectBadgeRouter(r *mux.Router) {
	r.HandleFunc("/{name}/{metric}", getProjectBadge).Methods("GET")
	r.HandleFunc("/{name}", setProjectBadge).Methods("POST")
}
